// FileTreeComponent
// Presents files and directories from the JFS server using a windows-explorer
// style hierarchy.
import java.awt.*;
import java.io.*;
import java.applet.Applet;
import java.util.Vector;
import java.util.Date;
import java.util.StringTokenizer;

public class FileTreeComponent extends JFScomponent
	implements HierarchyCallback, MultiColumnCallback
{
	FileTreeNode root;			// root of files displayed
	Hierarchy tree;				// the hierarchy widget
	MultiColumn dlist, vlist;		// directory and versions list
	Panel card;
	Button runb, openb, refb, fileb, newb, copyb;
	StaticTextField namef, typef, sizef;

	// connect
	// Build a tree structure down to the user's home directory from the
	// JFS server.
	void connect(JFSclient c)
	{
	super.connect(c);
	try { FileTreeNode.init(this); }
	catch(RequestException e) {
		new ErrorWindow("Could not load icons : "+e.getMessage());
		}
	try {
		root = new FileTreeNode(c, "/");
		root.expand(c);
		root.open = true;
		root.text = "/";
		StringTokenizer tok = new StringTokenizer(c.currentdir, "/");
		FileTreeNode pa = root;
		while(tok.hasMoreTokens()) {
			FileTreeNode ch = pa.find(tok.nextToken());
			ch.expand(c);
			ch.open = true;
			pa = ch;
			}
		}
	catch(RequestException e) {
		new ErrorWindow("Error opening home directory : "+
				e.getMessage());
		}
	}

	// init
	// Create the UI
	void init()
	{
	setLayout(new BorderLayout());
	Panel cen = new Panel();
	cen.setLayout(new GridLayout(1, 2));
	cen.add(tree = new Hierarchy(root, this));
	card = new Panel();
	card.setLayout(new CardLayout());
	String dtitles[] = { "Name", "Type" };
	card.add("dir", dlist = new MultiColumn(dtitles, this));
	String vtitles[] = { "Version", "Size", "Creator" };
	card.add("ver", vlist = new MultiColumn(vtitles, this));
	cen.add(card);
	add("Center", cen);
	Panel top = new Panel();
	top.setLayout(new FlowLayout(FlowLayout.LEFT));
	top.add(runb = new Button("Run"));
	top.add(openb = new Button("Open"));
	top.add(refb = new Button("Refresh"));
	top.add(fileb = new Button("File"));
	top.add(newb = new Button("New"));
	top.add(copyb = new Button("Copy"));
	add("North", top);
	}


	// wantedsize
	// How big would this component like to be?
	Dimension wantedsize()
	{
	return new Dimension(400, 500);
	}

	// action
	// Handle user input from a button
	public boolean action(Event evt, Object obj)
	{
	if (evt.target == runb) {
		// Open requestor for program to run
		reg(new ProgramChooser(client));
		}
	else if (evt.target == openb) {
		// Open the currently selected file
		FileTreeNode s = (FileTreeNode)tree.selected();
		if (s != null)
			open(s.path, s.fl, s.vnum);
		}
	else if (evt.target == refb) {
		// Refresh the currenty selected directory
		FileTreeNode s = (FileTreeNode)tree.selected();
		if (s != null && s.type == 0) {
			try { s.expand(client); }
			catch(RequestException e) {
				new ErrorWindow("Error refreshing directory : "+
						e.getMessage());
				}
			tree.redraw();
			}
		}
	else if (evt.target == fileb) {
		// Open window with file details
		FileTreeNode s = (FileTreeNode)tree.selected();
		if (s != null)
			reg(new FileInfoReq(this, s.fl, s.path));
		}
	return true;
	}

	// openNode
	// Called when a directory node is opened.
	public void openNode(Hierarchy h, HierarchyNode n)
	{
	}

	// closeNode
	// Called when a directory node is closed
	public void closeNode(Hierarchy h, HierarchyNode n)
	{
	}

	// clickNode
	// Called upon a mouse click on a file or directory node. If the
	// chosen file is a directory, display it's contents in the list box
	// to the right. Otherwise, display version details.
	public void clickNode(Hierarchy h, HierarchyNode n)
	{
	FileTreeNode nd = (FileTreeNode)n;
	JFSfile fl = nd.fl;
	if (nd.type == 0) {
		// Switch to dir list and display files
		if (!nd.expanded) {
			try { nd.expand(client); }
			catch(RequestException e) {
				new ErrorWindow("Error reading "+nd.path+" : "+
						e.getMessage());
				return;
				}
			}
		((CardLayout)card.getLayout()).show(card, "dir");
		dlist.clear();
		for(int i=0; i<nd.size(); i++) {
			FileTreeNode dnd = nd.find(i);
			String tmp[] = { dnd.fl.name, dnd.fl.type };
			dlist.addItem(tmp);
			}
		}
	else if (nd.ch == null) {
		// Switch to versions list and display a single-version file
		((CardLayout)card.getLayout()).show(card, "ver");
		vlist.clear();
		JFSversion vi = (JFSversion)nd.fl.versions.elementAt(0);
		String tmp[] = { "Single", String.valueOf(vi.size),
				 vi.creator };
		vlist.addItem(tmp);
		}
	else {
		// Switch to versions list and display a multi-version file
		((CardLayout)card.getLayout()).show(card, "ver");
		vlist.clear();
		for(int i=0; i<nd.size(); i++) {
			JFSversion vi = null;
			int vnum = nd.find(i).vnum;
			for(int j=0; j<nd.fl.versions.size(); j++) {
				JFSversion vr = (JFSversion)nd.fl.versions.
							    elementAt(j);
				if (vr.number == vnum)
					vi = vr;
				}
			String tmp[] = { String.valueOf(vnum),
					 String.valueOf(vi.size),
					 vi.creator };
			vlist.addItem(tmp);
			}
		}
	}

	// doubleNode
	// Called when a single-version file or version is double-clicked on.
	// Run the appropriate handler class to view this file.
	public void doubleNode(Hierarchy h, HierarchyNode n)
	{
	FileTreeNode nd = (FileTreeNode)n;
	open(nd.path, nd.fl, nd.vnum);
	}

	// singleClick
	// Called upon a click on the directory or versions list
	public void singleClick(MultiColumn m, int n)
	{
	}

	// doubleClick
	// Called upon a double-click in the directory or versions list
	public void doubleClick(MultiColumn m, int n)
	{
	FileTreeNode sel = (FileTreeNode)tree.selected();
	FileTreeNode nd = sel.ch!=null ? sel.find(n) : null;
	if (m == dlist) {
		// Double-click on a file or directory. Open the latest version
		if (nd.type == 0) {
			// directory.. open / close it
			nd.open = !nd.open;
			if (nd.open) {
				// 'fake' a click on this node in the tree
				sel.open = true;
				clickNode(tree, nd);
				tree.select(nd);
				}
			tree.redraw();
			}
		else {
			// file.. open the highest version
			int max = 0;
			for(int i=0; i<nd.fl.versions.size(); i++) {
				int vn = ((JFSversion)nd.fl.versions.
						      elementAt(i)).number;
				if (vn > max) max = vn;
				}
			open(nd.path, nd.fl, nd.fl.multiversion ? max : 0);
			}
		}
	else {
		// Double-click on a version. Open it
		if (sel.fl.multiversion)
			open(nd.path, nd.fl, nd.vnum);
		else
			open(sel.path, sel.fl, 0);
		}
	}

	// open
	// Find and use the corrent handler class for the given file.
	void open(String path, JFSfile file, int ver)
	{
	String handler = null;
	try { handler = client.gethandler(file.type); }
	catch(RequestException e) {
		new ErrorWindow("Error getting handler for "+file.type+" : "+
				e.getMessage());
		return;
		}
	if (handler != null)
		new ProgramWindow(handler, client, path, ver);
	}
}


// FileTreeNode
// One node in the tree of files and directories being displayed. Each node
// can be either a directory, a file or a version of a file. A directory
// node can either be listed or empty, depending on whether the contents
// of that directory have been received from the server yet.
class FileTreeNode extends HierarchyNode
{
	static Vector icont = new Vector();
	static Vector iconi = new Vector();
	JFSfile fl;			// information about this file, or null
					// for a version node.
	int type;			// 0=dir, 1=file, 2=version
	String path;			// the full path to this file
	int vnum;			// version number if type = 2
	boolean expanded = false;	// has expand() been called?

	// init
	// Initialize the mapping between MIME types and icons from the file
	// /etc/icons by reading the needed images
	static void init(FileTreeComponent p) throws RequestException
	{
	BufferInputStream in = new BufferInputStream(
				p.client.get("/etc/icons", 0));
	MediaTracker mt = new MediaTracker(p);
	boolean defaulticon = false;
	try {
		while(true) {
			String line = in.gets();
			StringSplitter tok = new StringSplitter(line, ':');
			if (tok.countTokens() != 2)
				continue;
			MimeType nt = new MimeType(tok.nextToken());
			if (nt.toString().equals("*/*"))
				defaulticon = true;
			Image ni = p.loadimage(tok.nextToken(), 0);
			mt.addImage(ni, 0);
			icont.addElement(nt);
			iconi.addElement(ni);
			}
		}
	catch(IOException e)
		{ }	// eof
	try { mt.waitForAll(); }
	catch(InterruptedException e) { }
	if (mt.isErrorAny())
		throw new RequestException("Error loading icon");
	if (!defaulticon)
		throw new RequestException("No default icon defined!");
	}

	// Create a new node by reading from the server information about
	// the given file or directory. If this node is a directory, the
	// contents will not be read until the expand() method is called.
	FileTreeNode(JFSclient c, String p) throws RequestException
	{
	this(c.statfile(p), p, 0);
	}

	// Create a new node for a given file structure and path. If v is
	// non-zero, then this is a version node, otherwise f determines if
	// it is a file or directory.
	FileTreeNode(JFSfile f, String p, int v)
	{
	path = p;
	vnum = v;
	fl = f;
	im = iconfor(f.type);
	if (vnum != 0) {
		// Version file node
		type = 2;
		text = "Version "+vnum;
		}
	else if (fl.type.equals("jfs/directory")) {
		// Directory node
		type = 0;
		text = f.name+"/";
		ch = new Vector();
		}
	else {
		// File node
		type = 1;
		text = f.name;
		if (fl.multiversion) {
			// Multi-version file node
			ch = new Vector();
			for(int i=0; i<fl.versions.size(); i++) {
				JFSversion vr =
					(JFSversion)fl.versions.elementAt(i);
				ch.addElement(
				 new FileTreeNode(fl, path, vr.number));
				}
			}
		}
	}

	// iconfor
	// Finds the icon for a given MIME type
	Image iconfor(String t)
	{
	MimeType mt = new MimeType(t);
	for(int i=0; i<icont.size(); i++)
		if (((MimeType)icont.elementAt(i)).equals(mt))
			return (Image)iconi.elementAt(i);
	return null;	// should never happen
	}

	// expand
	// For a directory, read the contents of this directory and add
	// them to the tree as it's children
	void expand(JFSclient c) throws RequestException
	{
	JFSdirectory d = c.getdir(path);
	ch.removeAllElements();
	for(int i=0; i<d.size(); i++) {
		JFSfile f = d.find(i);
		String sep = path.endsWith("/") ? "" : "/";
		ch.addElement(new FileTreeNode(f, path+sep+f.name, 0));
		}
	expanded = true;
	}

	// find
	// Find the node for the named file under this one
	FileTreeNode find(String name)
	{
	for(int i=0; i<ch.size(); i++) {
		FileTreeNode n = (FileTreeNode)ch.elementAt(i);
		if (n.fl.name.equals(name))
			return n;
		}
	return null;
	}

	// find
	// Returns the Nth child of this node
	FileTreeNode find(int num)
	{
	return (FileTreeNode)ch.elementAt(num);
	}

	// size
	// How many children of this node?
	int size()
	{
	return ch.size();
	}
}


// FileInfoReq
// Displays information about a file or directory, and allows deleting,
// renaming and changing of the MIME type.
class FileInfoReq extends FixedFrame implements MultiColumnCallback
{
	FileTreeComponent parent;
	JFSclient client;
	JFSfile file;
	String path;
	TextField filename;
	TextField mimetype;
	Button perms;
	MultiColumn vlist, plist;
	Button closeb, delb, renb, typeb;

	FileInfoReq(FileTreeComponent t, JFSfile f, String p)
	{
	super(400, 300);
	parent = t;
	client = t.client;
	file = f;
	path = p;

	// create UI
	setLayout(new BorderLayout());

	Panel top = new Panel();
	top.setLayout(new BorderLayout());
	Panel left = new Panel();
	left.setLayout(new GridLayout(2, 1));
	left.add(new Label("Filename"));
	left.add(new Label("MIME type"));
	top.add("West", left);
	Panel right = new Panel();
	right.setLayout(new GridLayout(2, 1));
	right.add(filename = new TextField(path));
	right.add(mimetype = new TextField(file.type));
	if (file.type.equals("jfs/directory"))
		mimetype.setEditable(false);
	top.add("Center", right);
	add("North", top);

	Panel mid = new Panel();
	mid.setLayout(new GridLayout(2, 1));
	Panel ppan = new Panel();
	ppan.setLayout(new BorderLayout());
	ppan.add("North", new Label("Permissions"));
	String ptitles[]= { "User/group", "Access" };
	ppan.add("Center", plist = new MultiColumn(ptitles, this));
	mid.add(ppan);

	Panel vpan = new Panel();
	vpan.setLayout(new BorderLayout());
	vpan.add("North", new Label("Versions"));
	String vtitles[] = { "Version", "Size", "Creator" };
	vpan.add("Center", vlist = new MultiColumn(vtitles, this));
	mid.add(vpan);
	add("Center", mid);

	Panel bot = new Panel();
	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
	bot.add(closeb = new Button("Close"));
	bot.add(delb = new Button("Delete"));
	bot.add(renb = new Button("Rename"));
	bot.add(typeb = new Button("Change type"));
	add("South", bot);
	pack();
	show();
	setTitle(file.name);

	// Fill in versions and permissions
	vfill();
	pfill();
	}

	// vfill
	// Fill the versions list
	void vfill()
	{
	vlist.clear();
	for(int i=0; i<file.versions.size(); i++) {
		JFSversion v = (JFSversion)file.versions.elementAt(i);
		String row[] = { file.multiversion ? String.valueOf(v.number)
						   : "Single",
				 String.valueOf(v.size),
				 v.creator };
		vlist.addItem(row);
		}
	}

	// pfill
	// Fill the permissions list
	void pfill()
	{
	plist.clear();
	for(int i=0; i<file.users.size(); i++) {
		JFSfileuser f = (JFSfileuser)file.users.elementAt(i);
		String row[] = { f.name, f.rwp };
		plist.addItem(row);
		}
	}

	// singleClick
	public void singleClick(MultiColumn list, int num) { }

	// doubleClick
	// Open a window with version information.
	public void doubleClick(MultiColumn list, int num)
	{
	}
}

