// ImageLoader
// This is an abstract class that implements some of the basic methods
// needed to create an Image object from image data in some format.
// Subclasses should override one (or both) of the input() methods
// to read image format data and store it into the data[][] array in
// 32-bit ARGB format. Other classes can then create an Image object
// from the data by doing something like..
//  ImageLoader l = new XXXloader();
//  l.input(somestream);
//  Image i = createImage(l);
import java.awt.image.*;
import java.io.*;
import java.util.Vector;

public abstract class ImageLoader implements ImageProducer
{
	int data[][];			// where the raw data should go.
					// This array should be [height][width]
	int w, h;			// image size
	boolean error;			// did something go wrong reading?

	Vector cons = new Vector();
	ColorModel cm = new DirectColorModel(24, 0xff0000, 0x00ff00, 0x0000ff);

	// input
	// Read image data from the given stream
	public void input(InputStream i)
	{
	ByteArrayOutputStream buf = new ByteArrayOutputStream();
	byte a[] = new byte[1024];
	int n;
	try { while((n = i.read(a)) > 0) buf.write(a, 0, n); }
	catch(IOException e) { }
	input(buf.toByteArray());
	}

	// input
	// Read image data from the given array
	public void input(byte b[])
	{
	input(new ByteArrayInputStream(b));
	}

	// produce
	// Does the actual work of sending data to the current list of
	// consumers.
	void produce()
	{
	for(int i=0; i<cons.size(); i++) {
		ImageConsumer c = (ImageConsumer)cons.elementAt(i);
		if (error) {
			c.imageComplete(ImageConsumer.IMAGEERROR);
			continue;
			}
		c.setDimensions(w,h);
		c.setColorModel(cm);
		c.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
		for(int j=0; j<h; j++)
			c.setPixels(0, j, w, 1, cm, data[j], 0, w);
		c.imageComplete(ImageConsumer.STATICIMAGEDONE);
		}
	}

	public void startProduction(ImageConsumer ic)
	{
	cons.addElement(ic);
	produce();
	}

	public void addConsumer(ImageConsumer ic)
	{
	cons.addElement(ic);
	}

	public boolean isConsumer(ImageConsumer ic)
	{
	return cons.contains(ic);
	}

	public void removeConsumer(ImageConsumer ic)
	{
	//cons.removeElement(ic);
	}

	public void requestTopDownLeftRightResend(ImageConsumer ic)
	{
	cons.addElement(ic);
	produce();
	}
}

