// ImageToggle.java
// A toggle-able button with an image on it. Works a lot like a checkbox.
import java.awt.*;
import java.util.Vector;

public class ImageToggle extends Canvas
{
	ImageToggleGroup group;	// mutex group to which this button belongs
	Image im1, im2;		// non-indented and indented images
	int border;		// pixel size of 3d border
	Color col1, col2;	// colours for border
	boolean state;		// are we indented?
	int w,h;		// max image size

	ImageToggle(Image i, boolean s, ImageToggleGroup g)
	{
	this(i, i, 3, s, g);
	}

	ImageToggle(Image i, int b, boolean s, ImageToggleGroup g)
	{
	this(i, i, b, s, g);
	}

	ImageToggle(Image i1, Image i2, boolean s, ImageToggleGroup g)
	{
	this(i1, i2, 3, s, g);
	}
	
	ImageToggle(Image i1, Image i2, int b, boolean s, ImageToggleGroup g)
	{
	im1 = i1; im2 = i2;
	border = b;
	group = g;
	state = s;

	// Wait for the images to properly load
	MediaTracker wt = new MediaTracker(this);
	wt.addImage(im1, 666);
	wt.addImage(im2, 666);
	try { wt.waitForAll(); }
	catch(InterruptedException e) { }
	w = Math.max(im1.getWidth(this), im2.getWidth(this));
	h = Math.max(im1.getHeight(this), im2.getHeight(this));

	// alloc colours
	col1 = new Color(220, 220, 220);
	col2 = new Color(50, 50, 50);

	// register with group
	group.add(this, state);
	}

	// paint
	// Re-render this component
	public void paint(Graphics g)
	{
	if (g == null) return;
	int w = size().width, h = size().height;
	g.setColor(state?col2:col1);
	for(int i=0; i<border; i++) {
		g.drawLine(i,i,w-i,i);
		g.drawLine(i,i,i,h-i);
		}
	g.setColor(state?col1:col2);
	for(int i=0; i<border; i++) {
		g.drawLine(w-i,h-i, w-i,i);
		g.drawLine(w-i,h-i, i,h-i);
		}

	// Paint image
	g.drawImage(state?im2:im1, border, border, w-border*2, h-border*2,this);
	}

	// mouseDown
	// Indent this button (if it isn't already)
	public boolean mouseDown(Event evt, int x, int y)
	{
	if (state != true) {
		state = true;
		group.notify(this);
		paint(getGraphics());
		getParent().postEvent(new Event(this,Event.ACTION_EVENT,null));
		}
	return true;
	}

	// unset
	// Called by group to unindent this button
	void unset()
	{
	state = false;
	paint(getGraphics());
	}

	public Dimension minimumSize()
	{
	return new Dimension(w+border*2, h+border*2);
	}

	public Dimension preferredSize()
	{
	return minimumSize();
	}
}


// ImageToggleGroup
// Like a CheckboxGroup, but for ImageToggle's
class ImageToggleGroup
{
	Vector members = new Vector();

	// add
	// Add an ImageToggle to this group. If it is set, then unset
	// every other one.
	void add(ImageToggle t, boolean s)
	{
	if (s) 
		for(int i=0; i<members.size(); i++)
			((ImageToggle)members.elementAt(i)).unset();
	members.addElement(t);
	}

	// notify
	// Unset every toggle except the calling one
	void notify(ImageToggle t)
	{
	for(int i=0; i<members.size(); i++) {
		ImageToggle o = (ImageToggle)members.elementAt(i);
		if (o != t) o.unset();
		}
	}
}

