// PPMloader
// A quick and dirty ImageProducer for loading raw format PPM data from
// an input stream.
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.util.Vector;
import LineInputStream;

public class PPMloader extends ImageLoader
{
	LineInputStream in;

	// input
	// Read from a stream
	public void input(InputStream is)
	{
	in = new LineInputStream(is);
	// read header
	boolean raw;
	String magic = ppmgets();
	if (magic.equals("P6"))
		raw = true;
	else if (magic.equals("P3"))
		raw = false;
	else {
		error = true;
		return;
		}
	String size = ppmgets();
	w = Integer.parseInt(size.substring(0,size.indexOf(' ')));
	h = Integer.parseInt(size.substring(size.indexOf(' ')+1));
	data = new int[h][w];
	String max = ppmgets();
	if (Integer.parseInt(max) != 255) {
		error = true;
		return;
		}

	try {
		if (raw) {
			// read raw data
			byte brow[] = new byte[w*3];
			for(int i=0; i<h; i++) {
				if (in.readdata(brow) != w*3)
					throw new IOException("Short read");
				for(int j=0, k=0; j<w*3; j+=3) {
					int r=brow[j], g=brow[j+1], b=brow[j+2];
					r = r<0 ? r+256 : r;
					g = g<0 ? g+256 : g;
					b = b<0 ? b+256 : b;
					data[i][k++] = (r<<16) + (g<<8) + b;
					}
				}
			}
		else {
			// read ascii data
			for(int i=0; i<h; i++) {
				for(int j=0; j<w; j++) {
					int r = Integer.parseInt(in.getw());
					int g = Integer.parseInt(in.getw());
					int b = Integer.parseInt(in.getw());
					data[i][j] = (r<<16) + (g<<8) + b;
					}
				}
			}
		in.close();
		}
	catch(IOException e) {
		error = true;
		return;
		}
	}


	// ppmgets
	// Read one line from PPM format image data, ignoring comments
	String ppmgets()
	{
	String line;
	do {
		try { line = in.gets(); }
		catch(IOException e) {
			return "";
			}
		} while(line.charAt(0) == '#');
	return line;
	}
}

